import matplotlib.pyplot as plt
import numpy as np
import deepdish as dd
from matplotlib import rcParams
import corner
import glob
from scipy.integrate import cumtrapz

results_dir = '../Multiple-Fig-Data/mass_ppd/'
map_redshift_to_names = {'NoEvolution': '_fixed_redshift/', 'Evolution': '/'}
map_vt_to_names = {'FullVT': '_full_vt', 'LowVT': '_low_vt'}
post_fname = '_iid_mag_two_comp_iid_tilt_powerlaw_redshift_mass_data.h5'

map_mass_model_to_names = {'truncated' : 'mass_b', 'peak' : 'mass_c', 'BPL' : 'mass_d', 'multipeak': 'mass_e'}

map_BBHsel_to_names = {
    'NoAugNoEvolution': 'default',
    'NoMayNoEvolution': 'no_190521',
    'AllNoEvolution': 'all_events',
}

def path_to_file(BBHsel_name, mass_model):
    mass = map_mass_model_to_names[mass_model]
    BBHsel = map_BBHsel_to_names[BBHsel_name]
    return results_dir + BBHsel + '/o1o2o3_' + mass + post_fname


stat_file =  open("m1_m99_credible90_interval_stats.txt","w")

stat_dict = {}
for BBHsel_name in ['NoAugNoEvolution', 'NoMayNoEvolution', 'AllNoEvolution']:
    for mass_model in map_mass_model_to_names.keys():
        f = dd.io.load(path_to_file(BBHsel_name, mass_model))
        ppd = f["ppd"]
        lines = f["lines"]

        mass_1 = np.linspace(2, 100, 1000)
        mass_1_ppd_lines = lines["mass_1"]

        print("Calculating m99 for "+path_to_file(BBHsel_name, mass_model))
        
        m99 = []
        
        for m in mass_1_ppd_lines:                                                                                                                                                                        
            i = len(m)
            cumulative_prob = cumtrapz(m, mass_1, initial = 0)
            init_prob = cumulative_prob[-1]
            prob = init_prob
            final_prob = init_prob*0.99                                                                                                                                              
            while prob > (final_prob):
                i -= 1
                prob = cumulative_prob[i]                                                                                                                                                         
            m99.append(mass_1[i])

        m99 = np.array(m99)
        qm99 = [0.05,0.5,0.95]
        qtm99 = corner.quantile(m99,qm99)

        print("Calculating m1 for "+path_to_file(BBHsel_name, mass_model))

        m1 = []
        for m in mass_1_ppd_lines:
            i = 0
            cumulative_prob = cumtrapz(m, mass_1, initial = 0)
            init_prob = cumulative_prob[-1]
            prob = 0
            final_prob = init_prob*0.01
            while prob < (final_prob):
                i += 1
                prob = cumulative_prob[i]
            m1.append(mass_1[i])

        m1 = np.array(m1)
        qm1 = [0.05,0.5,0.95]
        qtm1 = corner.quantile(m1,qm1)
        
        vals_m1 = [round(qtm1[1], 1), round((qtm1[0]-qtm1[1]), 1), round((qtm1[2]-qtm1[1]), 1)]
        vals_m99 = [round(qtm99[1], 1), round((qtm99[0]-qtm99[1]), 1), round((qtm99[2]-qtm99[1]), 1)]

        stat_dict[mass_model+BBHsel_name+"MOnePercentile"] = [
            'm1 percentile median and 90\% credible interval for '+path_to_file(BBHsel_name, mass_model),
            vals_m1[0],
            vals_m1[2],
            vals_m1[1],
        ]
        stat_dict[mass_model+BBHsel_name+"MNinetyNinePercentile"] = [
            'm99 percentile median and 90\% credible interval for '+path_to_file(BBHsel_name, mass_model),
            vals_m99[0],
            vals_m99[2],
            vals_m99[1],
        ]

for key in stat_dict:
    stat_file.write("% "+stat_dict[key][0]+"\n")
    stat_file.write("\\newcommand{\\%s}{\ensuremath{%s^{+%s}_{%s}}}\n" % (key, stat_dict[key][1],stat_dict[key][2],stat_dict[key][3]))
    stat_file.write(" \n")

stat_file.close()
